[][src]Struct dockertest::waitfor::RunningWait

pub struct RunningWait {
    pub check_interval: u64,
    pub max_checks: u64,
}

The RunningWait WaitFor implementation for containers. This variant will wait until the docker daemon reports the container as running.

Fields

check_interval: u64

How many seconds shall there be between each check for running state.

max_checks: u64

The number of checks to perform before erroring out.